@@ -74,7 +74,7 @@ sub checkfile {
 	my $this=shift;
 	my $file=shift;
 
-	return $file =~ m/.*\.deb$/;
+	return $file =~ m/.*\.u?deb$/;
 }
 
 =item install
@@ -397,14 +397,14 @@ build:
 clean:
 	dh_testdir
 	dh_testroot
-	dh_clean
+	dh_clean -d
 
 binary-indep: build
 
 binary-arch: build
 	dh_testdir
 	dh_testroot
-	dh_clean -k
+	dh_clean -k -d
 	dh_installdirs
 
 	dh_installdocs
